/*
 * author: Darren Bounds <darren@sfhn.net>
 * copyright: Copyright (C) 2002 by Darren Bounds
 * license: This software is under GPL version 2 of license
 *
 * packit official page at http://www.intrusense.com/~dbounds
 */

#include <libnet.h>
#include <pcap.h>
#include <sys/types.h>
#include <net/bpf.h>

#define MAX_CMDOPT_LEN         32

#ifndef _PACKIT_HEADERS
#define _PACKIT_HEADERS

#define PACKIT_ARP_H           0x1c    /* ARP header:          28 bytes */
#define PACKIT_DNS_H           0xc     /* DNS header base:     12 bytes */
#define PACKIT_ETH_H           0xe     /* Etherner header:     14 bytes */
#define PACKIT_ICMPV4_H        0x4     /* ICMP header base:     4 bytes */
#define PACKIT_ICMPV6_H        0x08    /* ICMP6 header base:    8 bytes */
#define PACKIT_ICMPV4_ECHO_H     0x8     /* ICMP_ECHO header:     8 bytes */
#define PACKIT_ICMPV4_MASK_H     0xc     /* ICMP_MASK header:    12 bytes */
#define PACKIT_ICMPV4_UNREACH_H  0x8     /* ICMP_UNREACH header:  8 bytes */
#define PACKIT_ICMPV4_TIMXCEED_H 0x8     /* ICMP_TIMXCEED header: 8 bytes */
#define PACKIT_ICMPV4_REDIRECT_H 0x8     /* ICMP_REDIRECT header: 8 bytes */
#define PACKIT_ICMPV4_TS_H       0x14    /* ICMP_TIMESTAMP headr:20 bytes */
#define PACKIT_IPV4_H          0x14    /* IP header:           20 bytes */
#define PACKIT_IPV6_H          0x28    /* IPv6 header:         40 bytes */
#define PACKIT_TCP_H           0x14    /* TCP header:          20 bytes */
#define PACKIT_UDP_H           0x8     /* UDP header:           8 bytes */

#endif /* _PACKIT_HEADERS */

u_short count;
u_short interval;
u_short display;
u_short linklayer;
u_short resolve;
u_short verbose;
u_short payload_s;

char *payload;
char *device;

char write_filename[MAX_CMDOPT_LEN];
char read_filename[MAX_CMDOPT_LEN];

struct iphdr_opts
{
    u_short protocol;		
    u_short version;			/* ip header version */
    u_short rand_saddr;      		 

    char *saddr;
    char *daddr;

    /* IPv4 header options */
    u_long v4saddr_nbo; 		/* ipv4 src address network byte order */ 
    u_long v4daddr_nbo; 		/* ipv4 dst address network byte order */

    u_short id;       			/* ipv4 id number */
    u_short ttl;	 		/* ipv4 time to live */

    long tos;			/* ipv4 type of service */

    /* IPv6 header options */
    struct libnet_in6_addr v6saddr_nbo; /* ipv6 src address network byte order */
    struct libnet_in6_addr v6daddr_nbo; /* ipv6 dst address network byte order */

    u_short hl;                 	/* ipv6 hop length */
    u_short plen;		 	/* ipv6 payload length */
    u_short fl;				/* flow label */
    u_short pri;			/* priority */

} iphdr_o;

struct tcphdr_opts
{
    u_short th_sport;		/* tcp src port */
    u_short th_dport;		/* tcp dst port */

    u_short th_urg;		/* tcp urg flag */
    u_short th_ack;		/* tcp ack flag */ 
    u_short th_psh;		/* tcp psh flag */
    u_short th_rst;		/* tcp rst flag */
    u_short th_syn;		/* tcp syn flag */
    u_short th_fin;		/* tcp fin flag */

    u_short th_urp;		/* tcp urg pointer */
    u_short th_win; 		/* tcp window size */	
    u_long th_ackn;		/* tcp ack number */
    u_long th_seqn;		/* tcp seq number */
} thdr_o;

struct udphdr_opts
{
    u_short uh_sport;   	/* udp src port */
    u_short uh_dport;		/* udp dst port */
    u_short uh_ulen;		/* udp length */
    u_short uh_sum;     	/* udp checksum */
} uhdr_o;

struct icmphdr_opts
{
    u_short ih_type;    	/* icmp4 type */
    u_short ih_code;		/* icmp4 code */ 
    u_short ih_id;		/* icmp4 id */ 
    u_short ih_seq;             /* icmp4 seq */
} ihdr_o;

struct enethdr_opts
{
    char *str_saddr;		/* source ethernet address string */
    char *str_daddr; 	  	/* destination ethernet address string */

    u_char eh_saddr[6];		/* source ethernet address */
    u_char eh_daddr[6];		/* destination ethernet address */ 
} ehdr_o; 

struct icmp_d
{
    char icmp_type[20];   
    int icmp_type_int;  

    char icmp_code[20];
    int icmp_code_int; 
};

#ifndef _CMD_FUNC
#define _CMD_FUNC

int             packit_parse_capture_options(int argc, char *argv[]);
int             packit_parse_tcp_reset_options(int argc, char *argv[]);
int             packit_parse_inject_options(int argc, char *argv[], int ip_version);

#endif /* _CMD_FUNC */

#ifndef _GLOBAL_FUNC
#define _GLOBAL_FUNC

void 		usage();
void 		enter_packet_foundry();

int             start_packet_capture(char *pfilter);
int             start_packet_analysis(char *pfilter);
int		start_packet_replay(char *pfilter);
int		packit_inject_options(int argc, char *argv[]);
int     	packit_retrieve_tcp_flags();
int		packit_build_ethernet_addr(char *ethstr, int sd);

void            display_tcp_data(u_char *packet, 
                    char *time_stamp,
                    struct libnet_ethernet_hdr *ehdr,
                    struct libnet_ipv4_hdr *iphdr);
void            display_udp_data(u_char *packet, 
                    char *time_stamp,
                    struct libnet_ethernet_hdr *ehdr,
                    struct libnet_ipv4_hdr *iphdr);
void            display_icmp_data(u_char *packet, 
                    char *time_stamp,
                    struct libnet_ethernet_hdr *ehdr,
                    struct libnet_ipv4_hdr *iphdr);

u_char *	packit_retrieve_rand_v4_addr();

pcap_t *	initialize_packet_capture(pcap_t *, char *);

libnet_t *	packit_build_tcp(libnet_t *packet_d);
libnet_t *	packit_build_udp(libnet_t *packet_d);
libnet_t *	packit_build_icmp(libnet_t *packet_d);

#endif /* _GLOBAL_FUNC */
