/*
 * author: Darren Bounds <darren@sfhn.net>
 * copyright: Copyright (C) 2002 by Darren Bounds
 * license: This software is under GPL version 2 of license
 *
 * packit official page at http://www.intrusense.com/~dbounds
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "../include/packit.h"
#include "../include/error.h"

#define READ_TIMEOUT 		500
#define DEFAULT_SNAP_LEN       	64

int hdrlen;

void
process_packet_data(u_char *user, struct pcap_pkthdr *pkthdr, u_char *packet)
{
    char time_stamp[64];
    struct libnet_ethernet_hdr *ehdr;
    struct libnet_ipv4_hdr *iphdr;

    if(display)
    {
        ehdr = (struct libnet_ethernet_hdr *)(packet);

        snprintf(time_stamp, 64, "%02d:%02d:%02d.%06u", 
            (pkthdr->ts.tv_sec % 86400) / 3600, 
            ((pkthdr->ts.tv_sec % 86400) % 3600) / 60, 
            (pkthdr->ts.tv_sec % 86400) % 40, 
            (unsigned)pkthdr->ts.tv_usec); 

        iphdr = (struct libnet_ipv4_hdr *)(packet + hdrlen);

        switch(iphdr->ip_p)
        {
            case IPPROTO_TCP: 
                display_tcp_data(packet, time_stamp, ehdr, iphdr);
                break;

            case IPPROTO_UDP: 
                display_udp_data(packet, time_stamp, ehdr, iphdr);
                break;

            case IPPROTO_ICMP: 
                display_icmp_data(packet, time_stamp, ehdr, iphdr);
                break;
        }

        return;
    }
}

int
start_packet_capture(char *pfilter)
{
    int dlink;
    struct bpf_program bpf;
    bpf_u_int32 localnet, netmask;
    pcap_t *p;
    pcap_dumper_t *pd;

    if(strlen(read_filename) > 0)
    {
        if((p = pcap_open_offline(read_filename, errbuf)) == NULL)
        {
            fprintf(stderr, "Error: Unable to open file: %s\n", errbuf); 
            exit(EXIT_FAILURE);
        }
    }
    else
    {
        if (device == NULL)
           if((device = pcap_lookupdev(errbuf)) == NULL)
           {
               fprintf(stderr, "Error: Unable to find appropriate device: %s\n", errbuf);
               exit(EXIT_FAILURE);
           }

        if((p = pcap_open_live(device, DEFAULT_SNAP_LEN, 1, READ_TIMEOUT, errbuf)) == NULL)
        {
            fprintf(stderr, "Error: Unable to open device: %s", errbuf);
            exit(EXIT_FAILURE);
        }
    }

    display = 1;

    if(strlen(write_filename) > 0)
    {
        if((pd = pcap_dump_open(p, write_filename)) == NULL)
        {
            fprintf(stderr, "Error: Unable to initialize packet capture: %s\n", pcap_geterr(p));
            exit(EXIT_FAILURE);
        }

        display--;
    }

    pcap_lookupnet(device, &localnet, &netmask, errbuf);

    if(pcap_compile(p, &bpf, pfilter, 0, netmask) < 0)
    {
        fprintf(stderr, "Error: Unable to process packet filters: %s", pcap_geterr(p));
        exit(EXIT_FAILURE);
    }

    if(pcap_setfilter(p, &bpf) < 0)
    {
        fprintf(stderr, "Error: Unable to set packet filters: %s", pcap_geterr(p));
        exit(EXIT_FAILURE);
    }

    if((dlink = pcap_datalink(p)) < 0)
    {
        fprintf(stderr, "Error: Unable to determine datalink type: %s", pcap_geterr(p));
        exit(EXIT_FAILURE);
    }

    switch(dlink)
    {
        case DLT_NULL:
            hdrlen = 4;
            break;

        case DLT_EN10MB:
            hdrlen = PACKIT_ETH_H; 
            break;

        default:
            hdrlen = PACKIT_ETH_H; 
            break;
    }

    fprintf(stderr, "mode: packet capture ");

    if(strlen(read_filename) > 0)
        fprintf(stderr, "using file: %s ", read_filename);
    else
        fprintf(stderr, "using device: %s ", device);

    if(pfilter)
        fprintf(stderr, "[%s]", pfilter);

    fprintf(stderr, "\n");

    pcap_loop(p, -1, (pcap_handler) process_packet_data, NULL);

    return 1;
}

