/*
 * author: Darren Bounds <darren@sfhn.net>
 * copyright: Copyright (c) 2002 by Darren Bounds
 * license: This software is under GPL version 2 of license
 * 
 * packit official page at http://www.intrusense.com/~dbounds
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "../include/packit.h"
#include "../include/error.h"
#include "../include/version.h"

int opt;
char *optarg;

int
packit_parse_capture_options(int argc, char *argv[])
{
    resolve = 1;

    while((opt = getopt(argc, argv, "ei:nr:vw:")) != -1)
    {
        switch(opt)
        {
            case 'e':
                linklayer = 1;
                break;

            case 'i':
                device = optarg;
                break;

            case 'w':
                strncpy(write_filename, optarg, MAX_CMDOPT_LEN);
                break;

            case 'r':
                strncpy(read_filename, optarg, MAX_CMDOPT_LEN);
                break;

            case 'v':
                verbose = 1;
                break;

            case 'n':
                resolve = 0;
                break;
        }
    }

    start_packet_capture(argv[optind]);
    return 1;
}

int
packit_parse_inject_options(int argc, char *argv[], int ip_version)
{
    count = 1;
    interval = 1;
    payload = NULL;
    payload_s = 0; 

    memset(&iphdr_o, 0, sizeof(iphdr_o));
    memset(&ehdr_o, 0, sizeof(iphdr_o));

    iphdr_o.ttl = 128;
    iphdr_o.tos = 0x00;
    iphdr_o.version = ip_version;

    ehdr_o.str_saddr = NULL;
    

    while((opt = getopt(argc, argv, "TUI")) != -1)
    {
        switch(opt)
        {
            case 'T':
                memset(&thdr_o, 0, sizeof(thdr_o));

                thdr_o.th_win = 1500;
                thdr_o.th_ack = NULL;
                iphdr_o.protocol = IPPROTO_TCP;
                
                while((opt = getopt(argc, argv, "a:c:d:D:e:E:F:h:i:n:p:q:rs:S:t:O:u:w:W:")) != -1)
                {
                    switch(opt)
                    {
                        case 'a':
                            thdr_o.th_ackn = (u_long)atoi(optarg);
                            break;

                        case 'c':
                            count = (u_short)atoi(optarg);
                            break;

                        case 'd':
                            iphdr_o.daddr = optarg;
                            break;

                        case 'D':
                            thdr_o.th_dport = (u_short)atoi(optarg);
                            break;

                        case 'e':
                            ehdr_o.str_saddr = optarg;
                            break;

                        case 'E':
                            ehdr_o.str_daddr = optarg;
                            break;

                        case 'F':
                            if(strrchr(optarg, 'U'))
                                thdr_o.th_urg = 1;

                            if(strrchr(optarg, 'A'))
                                thdr_o.th_ack = 1;

                            if(strrchr(optarg, 'P'))
                                thdr_o.th_psh = 1;

                            if(strrchr(optarg, 'R'))
                                thdr_o.th_rst = 1;

                            if(strrchr(optarg, 'S'))
                                thdr_o.th_syn = 1;

                            if(strrchr(optarg, 'F'))
                                thdr_o.th_fin = 1;

                            break;

                        case 'h':
                            iphdr_o.hl = (u_short)atoi(optarg);
                            break;

                        case 'i':
                            device = optarg;
                            break;

                        case 'n':
                            iphdr_o.id = (u_short)atoi(optarg);
                            break;

                        case 'O':
                            iphdr_o.tos = (long)atoi(optarg);
                            break;

                        case 'p':
                            payload = optarg; 
                            break;

                        case 'q':
                            thdr_o.th_seqn = (u_long)atoi(optarg);
                            break;

                        case 'r':
                            iphdr_o.rand_saddr = 1;
                            break;

                        case 's':
                            iphdr_o.saddr = optarg;
                            break;

                        case 'S':
                            thdr_o.th_sport = (u_short)atoi(optarg);
                            break;

                        case 't':
                            iphdr_o.ttl = (u_short)atoi(optarg);
                            break;

                        case 'u':
                            thdr_o.th_urp = (u_short)atoi(optarg);
                            break;

                        case 'w':
                            interval = (u_short)atoi(optarg);
                            break;

                        case 'W':
                            thdr_o.th_win = (u_short)atoi(optarg);
                            break;
                    }
                }

                break;

            case 'I':
                memset(&ihdr_o, 0, sizeof(ihdr_o));

                ihdr_o.ih_type = 8;	/* default to echo request */
                ihdr_o.ih_code = 0;
                ihdr_o.ih_id = libnet_get_prand(LIBNET_PRu16); 
                ihdr_o.ih_seq = libnet_get_prand(LIBNET_PRu16);
                iphdr_o.protocol = IPPROTO_ICMP;

                while((opt = getopt(argc, argv, "c:C:s:d:e:E:i:rn:N:O:p:w:K:")) != -1)
                {

                    switch(opt)
                    {
                        case 'c':
                            count = (u_short)atoi(optarg);
                            break;

                        case 'i':
                            device = optarg;
                            break;

                        case 'w':
                            interval = (u_short)atoi(optarg);
                            break;

                        case 'd':
                            iphdr_o.daddr = optarg;
                            break;

                        case 'e':
                            ehdr_o.str_saddr = optarg;
                            break;

                        case 'E':
                            ehdr_o.str_daddr = optarg;
                            break;

                        case 'Q':
                            ihdr_o.ih_seq = (u_short)atoi(optarg);
                            break;

                        case 'r':
                            iphdr_o.rand_saddr = 1;
                            break;

                        case 's':
                            iphdr_o.saddr = optarg;
                            break;

                        case 't':
                            iphdr_o.ttl = (u_short)atoi(optarg);
                            break;

                        case 'n':
                            iphdr_o.id = (u_short)atoi(optarg);
                            break;

                        case 'N':
                            ihdr_o.ih_id = (u_short)atoi(optarg);
                            break;

                        case 'O':
                            iphdr_o.tos = (long)atoi(optarg);
                            break;

                        case 'p':
                            payload = optarg;
                            break;

                        case 'C':
                            ihdr_o.ih_code = (u_short)atoi(optarg); 
                            break;

                        case 'K':
                            ihdr_o.ih_type = (u_short)atoi(optarg); 
                            break;
                    }
                }

                break;

            case 'U': 
                memset(&uhdr_o, 0, sizeof(uhdr_o));
                iphdr_o.protocol = IPPROTO_UDP;

                while((opt = getopt(argc, argv, "c:i:w:d:e:E:rs:n:S:D:")) != -1)
                {
                    switch(opt)
                    {
                        case 'c':
                            count = (u_short)atoi(optarg);
                            break;

                        case 'd':
                            iphdr_o.daddr = optarg;
                            break;

                        case 'D':
                            uhdr_o.uh_dport = (u_short)atoi(optarg);
                            break;

                        case 'e':
                            ehdr_o.str_saddr = optarg;
                            break;

                        case 'E':
                            ehdr_o.str_daddr = optarg;
                            break;

                        case 'i':
                            device = optarg;
                            break;

                        case 'r':
                            iphdr_o.rand_saddr = 1;
                            break;

                        case 's':
                            iphdr_o.saddr = optarg;
                            break;

                        case 'S':
                            uhdr_o.uh_sport = (u_short)atoi(optarg);
                            break;

                        case 't':
                            iphdr_o.ttl = (u_short)atoi(optarg);
                            break;

                        case 'n':
                            iphdr_o.id = (u_short)atoi(optarg);
                            break;

                        case 'O':
                            iphdr_o.tos = (long)atoi(optarg);
                            break;

                        case 'p':
                            payload = optarg;
                            break;

                        case 'w':
                            interval = (u_short)atoi(optarg);
                            break;
                    }
                }

                break;
        }
    }
 
    enter_packet_foundry();

    return 1;
}
