/*
 * author: Darren Bounds <darren@sfhn.net>
 * copyright: Copyright (C) 2002 by Darren Bounds
 * license: This software is under GPL version 2 of license
 *
 * packit official page at http://www.intrusense.com/~dbounds
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "../include/packit.h"
#include "../include/error.h"

int hdrlen;

void
display_tcp_data(u_char *packet, char *time_stamp, struct libnet_ethernet_hdr *ehdr, struct libnet_ipv4_hdr *iphdr)
{
    char flags[7];

    struct libnet_tcp_hdr *tcphdr;
    struct in_addr ip_src, ip_dst;

    bzero(&ip_src, sizeof(ip_src));
    bzero(&ip_dst, sizeof(ip_dst));
    memset(flags, 0, sizeof(flags));

    tcphdr = (struct libnet_tcp_hdr *)(packet + PACKIT_IPV4_H + hdrlen);

    if(tcphdr->th_flags & TH_SYN)
        strcat(flags, "S");

    if(tcphdr->th_flags & TH_FIN)
        strcat(flags, "F");

    if(tcphdr->th_flags & TH_RST)
        strcat(flags, "R");

    if(tcphdr->th_flags & TH_PUSH)
        strcat(flags, "P");

    fprintf(stderr, "%s tcp ", time_stamp);

    if(linklayer)
    {
        fprintf(stderr, "%x:%x:%x:%x:%x:%x > ",  
            ehdr->ether_shost[0], ehdr->ether_shost[1], ehdr->ether_shost[2], ehdr->ether_shost[3], ehdr->ether_shost[4], ehdr->ether_shost[5]); 

        fprintf(stderr, "%x:%x:%x:%x:%x:%x ", 
            ehdr->ether_dhost[0], ehdr->ether_dhost[1], ehdr->ether_dhost[2], ehdr->ether_dhost[3], ehdr->ether_dhost[4], ehdr->ether_dhost[5]);
    }            

    fprintf(stderr, "%s.%u > %s.%u: ", 
        libnet_addr2name4(iphdr->ip_src.s_addr, resolve), 
        htons(tcphdr->th_sport), 
        libnet_addr2name4(iphdr->ip_dst.s_addr, resolve), 
        htons(tcphdr->th_dport));

    if(strlen(flags) < 1)
        fprintf(stderr, ". ");
    else
        fprintf(stderr, "%s ", flags);

    if(tcphdr->th_seq > 0)
        fprintf(stderr, "%u ", ntohl(tcphdr->th_seq));

    if(tcphdr->th_flags & TH_ACK || tcphdr->th_ack > 0)
        fprintf(stderr, "ack %u ", ntohl(tcphdr->th_ack));

    fprintf(stderr, "win %d ", htons(tcphdr->th_win));

    if(tcphdr->th_flags & TH_URG)
        fprintf(stderr, "urg %d ", ntohs(tcphdr->th_urp)); 

    if(ntohs(iphdr->ip_off) & IP_DF)
        fprintf(stderr, "(DF) ");

    if(verbose)
        fprintf(stderr, "(ttl %d, id %d, tos 0x%x )", iphdr->ip_ttl, iphdr->ip_id, iphdr->ip_tos);

    fprintf(stderr, "\n");

    return;
}
